/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, screen */

/**
 * UI helper module.
 * Provides helper functions used in the user interface.
 *
 * @module helpers/ui
 * @requires {@link core/template}
 * @namespace helpers/ui
 * @memberof helpers
 */

define({
    name: 'helpers/ui',
    requires: [
        'core/template'
    ],
    def: function ui(template) {
        'use strict';

        /**
         * List of screen resolutions for which the virtual keyboard is not
         * supported.
         *
         * @private
         * @const {{width: number, height: number}[]}
         */
        var SMALL_SCREEN_RESOLUTIONS = [{
                width: 320,
                height: 320
            }, {
                width: 360,
                height: 360
            }],

            /**
             * Indicates whether the virtual keyboard is supported or not.
             *
             * @private
             * @type {boolean}
             */
            virtualKeyboardSupported = true;

        /**
         * Creates HTML content of the list and fills the specified "ul" element
         * with created content.
         * The content will be composed of "li" elements created using specified
         * template.
         *
         * @memberof helpers/ui
         * @public
         * @param {HTMLElement} list Element which will be filled with created
         * content.
         * @param {*[]} templateArgs List of objects. Each object will be
         * passed to the template module in order to create the "li" item.
         * @param {string} templateName Name of the template.
         */
        function fillList(list, templateArgs, templateName) {
            var result = '',
                i = 0;

            for (i = 0; i < templateArgs.length; i += 1) {
                result += template.get(templateName, templateArgs[i]);
            }

            list.innerHTML = result;
        }

        /**
         * Checks if the virtual keyboard is supported for the current screen
         * resolution and saves this information
         * in the "virtualKeyboardSupported" variable.
         *
         * @private
         */
        function initVirtualKeyboardSupported() {
            var i = 0,
                resolution = null;

            for (i = 0; i < SMALL_SCREEN_RESOLUTIONS.length; i += 1) {
                resolution = SMALL_SCREEN_RESOLUTIONS[i];
                if (resolution.width === screen.width &&
                    resolution.height === screen.height) {
                    virtualKeyboardSupported = false;
                    break;
                }
            }
        }

        /**
         * Returns true if the virtual keyboard is supported for the current
         * screen resolution, false otherwise.
         *
         * @memberof helpers/ui
         * @public
         * @returns {boolean}
         */
        function isVirtualKeyboardSupported() {
            return virtualKeyboardSupported;
        }

        /**
         * Initializes module.
         *
         * @memberof helpers/ui
         * @public
         */
        function init() {
            initVirtualKeyboardSupported();
        }

        return {
            init: init,
            fillList: fillList,
            isVirtualKeyboardSupported: isVirtualKeyboardSupported
        };
    }
});
