/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, tau, Event */

/**
 * Details view module.
 *
 * @module views/details
 * @requires {@link core/event}
 * @requires {@link helpers/ui}
 * @requires {@link models/soundProfiles}
 * @namespace views/details
 * @memberof views
 */

define({
    name: 'views/details',
    requires: [
        'core/event',
        'helpers/ui',
        'models/soundProfiles'
    ],
    def: function details(req) {
        'use strict';

        /**
         * Event module instance.
         *
         * @private
         * @type {Module}
         */
        var event = req.core.event,

            /**
             * Ui helper module instance.
             *
             * @private
             * @type {Module}
             */
            uiHelper = req.helpers.ui,

            /**
             * Sound profiles module instance.
             *
             * @private
             * @type {Module}
             */
            soundProfiles = req.models.soundProfiles,

            /**
             * Reference to sound types list.
             *
             * @private
             * @type {HTMLElement}
             */
            soundTypesList = null,

            /**
             * Displayed sound profile.
             *
             * @private
             * @type {object}
             */
            profile = null,

            /**
             * Copy of loaded sound profile. Used to reset the profile to
             * initial values.
             *
             * @private
             * @type {object}
             */
            originalProfile = null,

            /**
             * Reference to page header.
             *
             * @private
             * @type {HTMLElement}
             */
            pageHeader = null,

            /**
             * Map of TAU sliders elements, where key is sound types and value
             * is a slider related to a given sound type.
             *
             * @private
             * @type {object}
             */
            sliders = {},

            /**
             * Reference to reset button.
             *
             * @private
             * @type {HTMLElement}
             */
            resetButton = null,

            /**
             * Reference to "delete" button.
             *
             * @private
             * @type {HTMLElement}
             */
            deleteButton = null,

            /**
             * Reference to delete confirmation pop-up.
             *
             * @private
             * @type {HTMLElement}
             */
            deleteConfirmPopup = null,

            /**
             * Reference to "yes" button on the "confirm deletion" panel.
             *
             * @private
             * @type {HTMLElement}
             */
            confirmDeleteButton = null;

        /**
         * Loads sound types.
         *
         * @private
         */
        function loadSoundTypes() {
            var templateArguments = [],
                soundTypes = soundProfiles.getSoundTypes(),
                i = 0;

            for (i = 0; i < soundTypes.length; i += 1) {
                templateArguments.push({name: soundTypes[i]});
            }

            uiHelper.fillList(soundTypesList, templateArguments,
                'soundTypeItem');
        }

        /**
         * Updates slider of given type with proper value.
         *
         * @private
         * @param {string} soundType
         */
        function refreshSlider(soundType) {
            var slider = sliders[soundType];

            slider.value = profile[soundType] * 10;
            slider.dispatchEvent(new Event('change'));
        }

        /**
         * Loads sound profile.
         *
         * @private
         */
        function loadProfile() {
            var soundTypes = soundProfiles.getSoundTypes(),
                i = 0;

            pageHeader.innerText = profile.title;

            for (i = 0; i < soundTypes.length; i += 1) {
                refreshSlider(soundTypes[i]);
            }
        }

        /**
         * Clones current profile data object.
         * Returns cloned object.
         *
         * @private
         * @returns {object}
         */
        function cloneProfile() {
            var clone = {},
                soundTypes = soundProfiles.getSoundTypes(),
                soundType = null,
                i = 0;

            for (i = 0; i < soundTypes.length; i += 1) {
                soundType = soundTypes[i];
                clone[soundType] = profile[soundType];
            }

            return clone;
        }

        /**
         * Handles "views.main.profile.clicked" event.
         *
         * @private
         * @param {CustomEvent} ev
         */
        function onOpenedProfile(ev) {
            profile = soundProfiles.getProfile(ev.detail);
            originalProfile = cloneProfile();
            loadProfile();
            tau.changePage('#details');
        }

        /**
         * Handles click event on the reset button.
         *
         * @private
         */
        function onResetClicked() {
            var soundTypes = soundProfiles.getSoundTypes(),
                soundType = null,
                i = 0;

            for (i = 0; i < soundTypes.length; i += 1) {
                soundType = soundTypes[i];
                if (originalProfile[soundType] !== profile[soundType]) {
                    soundProfiles.updateProfile(profile.id, soundType,
                        originalProfile[soundType]);
                    refreshSlider(soundType);
                }
            }
        }

        /**
         * Registers event listeners for a given list item elements.
         *
         * @private
         * @param {HTMLElement} li
         */
        function bindListItemEvents(li) {
            var valueSpan = li.querySelector('span'),
                sliderInput = li.querySelector('input'),
                soundType = sliderInput.id;

            sliderInput.addEventListener('change',
                function updateSpan() {
                    valueSpan.innerText = (sliderInput.value * 10) + '%';
                    soundProfiles.updateProfile(profile.id,
                        soundType, sliderInput.value / 10);
                });
            valueSpan.innerText = (sliderInput.value * 10) + '%';
        }

        /**
         * Handles click event on the "delete" button.
         *
         * @private
         */
        function onConfirmDeleteClicked() {
            soundProfiles.deleteProfile(profile.id);

            deleteConfirmPopup.addEventListener(
                'popuphide',
                function onPopupHide() {
                    deleteConfirmPopup.removeEventListener('popuphide',
                        onPopupHide, false);
                    tau.back();
                },
                false
            );

            tau.closePopup();
        }

        /**
         * Initializes TAU slider widgets.
         *
         * @private
         */
        function initSliders() {
            var soundTypes = soundProfiles.getSoundTypes(),
                soundType = null,
                i = 0;

            for (i = 0; i < soundTypes.length; i += 1) {
                soundType = soundTypes[i];
                sliders[soundType] = document.getElementById(soundType);
            }
        }

        /**
         * Registers event listeners.
         *
         * @private
         */
        function bindEvents() {
            var listItems = soundTypesList.getElementsByTagName('li'),
                i = 0;

            for (i = 0; i < listItems.length; i += 1) {
                bindListItemEvents(listItems[i]);
            }

            resetButton.addEventListener('click', onResetClicked);
            deleteButton.addEventListener('click', function onClick() {
                tau.openPopup('#deleteProfilePopup');
            });
            confirmDeleteButton.addEventListener('click',
                onConfirmDeleteClicked);

            event.listeners({
                'views.main.profile.clicked': onOpenedProfile
            });
        }

        /**
         * Initializes module.
         *
         * @memberof views/details
         * @public
         */
        function init() {
            soundTypesList = document.getElementById('soundTypes');
            pageHeader = document.querySelector('#details h2');
            resetButton = document.getElementById('reset');
            deleteButton = document.getElementById('delete');
            confirmDeleteButton = document.getElementById('confirmDelete');
            deleteConfirmPopup = document.getElementById('deleteProfilePopup');
            loadSoundTypes();
            initSliders();
            bindEvents();
        }

        return {
            init: init
        };
    }
});
