/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, tau */

/**
 * Main view module.
 *
 * @module views/main
 * @requires {@link core/event}
 * @requires {@link helpers/ui}
 * @requires {@link models/soundProfiles}
 * @namespace views/main
 * @memberof views
 */

define({
    name: 'views/main',
    requires: [
        'core/event',
        'helpers/ui',
        'models/soundProfiles'
    ],
    def: function main(req) {
        'use strict';

        /**
         * Event module instance.
         *
         * @private
         * @type {Module}
         */
        var event = req.core.event,

            /**
             * Ui helper module instance.
             *
             * @private
             * @type {Module}
             */
            uiHelper = req.helpers.ui,

            /**
             * SoundProfiles module instance.
             *
             * @private
             * @type {Module}
             */
            soundProfiles = req.models.soundProfiles,

            /**
             * Reference to profiles list.
             *
             * @private
             * @type {HTMLElement}
             */
            profilesList = null,

            /**
             * Reference to 'add profile' button.
             *
             * @private
             * @type {HTMLElement}
             */
            addProfileButton = null,

            /**
             * Reference to the "ok" button from the "Add profile" pop-up.
             *
             * @private
             * @type {HTMLElement}
             */
            addButton = null,

            /**
             * Reference to the profile's name input field.
             *
             * @private
             * @type {HTMLElement}
             */
            nameInput = null,

            /**
             * Reference to the button on the message pop-up.
             *
             * @private
             * @type {HTMLElement}
             */
            hideMessage = null;

        /**
         * Handles click event on profiles list items.
         *
         * @private
         * @param {MouseEvent} ev
         * @fires views.main.profile.clicked
         */
        function onProfileClick(ev) {
            var id = parseInt(ev.currentTarget.getAttribute('data-profile-id'),
                10);

            event.fire('profile.clicked', id);
        }

        /**
         * Handles click event on "Set" buttons.
         *
         * @private
         * @param {MouseEvent} ev
         */
        function onSetProfileClick(ev) {
            var profileId = ev.currentTarget.getAttribute('data-profile-id');

            soundProfiles.setActive(parseInt(profileId, 10));

            ev.stopPropagation();
            ev.preventDefault();
        }

        /**
         * Creates HTML content of profiles list.
         *
         * @private
         */
        function createListContent() {
            var activeProfileId = soundProfiles.getActiveProfileId(),
                i = 0,
                profiles = soundProfiles.getProfiles(),
                profile = null,
                templateArguments = [];

            for (i = 0; i < profiles.length; i += 1) {
                profile = profiles[i];
                templateArguments.push({
                    title: profile.title,
                    id: profile.id,
                    isActive: profile.id === activeProfileId
                });
            }

            uiHelper.fillList(profilesList, templateArguments, 'listItem');
        }

        /**
         * Registers event listeners on profiles list.
         *
         * @private
         */
        function bindListEvents() {
            var liElements = profilesList.getElementsByTagName('li'),
                i = 0,
                li = null,
                setButton = null;

            for (i = 0; i < liElements.length; i += 1) {
                li = liElements[i];
                setButton = li.querySelector('input[type="radio"]');
                if (setButton) {
                    setButton.addEventListener('click', onSetProfileClick);
                }
                li.addEventListener('click', onProfileClick);
            }
        }

        /**
         * Loads profiles.
         *
         * @private
         */
        function loadProfiles() {
            createListContent();
            bindListEvents();
        }

        /**
         * Registers view event listeners.
         *
         * @private
         */
        function bindEvents() {
            addButton.addEventListener('click', function addProfileListener() {
                var name = nameInput.value.trim();

                if (name.length === 0) {
                    return;
                }

                if (soundProfiles.getProfileByTitle(name)) {
                    tau.openPopup('#messagePopup');
                } else {
                    soundProfiles.createProfile(name);
                    tau.closePopup();
                }
            });

            addProfileButton.addEventListener('click', function onClick() {
                if (uiHelper.isVirtualKeyboardSupported()) {
                    tau.openPopup('#addProfilePopup');
                } else {
                    soundProfiles.createProfile();
                }
            });

            hideMessage.addEventListener('click', function onClick() {
                tau.closePopup();
            });

            event.listeners({
                'models.soundProfiles.changed': loadProfiles
            });
        }

        /**
         * Initializes module.
         *
         * @memberof views/main
         * @public
         */
        function init() {
            profilesList = document.getElementById('profiles');
            addButton = document.getElementById('add');
            nameInput = document.getElementById('name');
            addProfileButton = document.getElementById('addProfileButton');
            hideMessage = document.getElementById('hideMessage');
            loadProfiles();
            bindEvents();
        }

        return {
            init: init
        };
    }
});
